//*****************************************************************************
//
// can_comm.h - Prototypes for the CAN communication functions.
//
// Copyright (c) 2008 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
// 
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 3618 of the RDK-BDC Firmware Package.
//
//*****************************************************************************

#ifndef __CAN_COMM_H__
#define __CAN_COMM_H__

//*****************************************************************************
//
// This function sends a system halt command.
//
//*****************************************************************************
#define CANSystemHalt()                                                       \
        CANSendMessage(CAN_MSGID_API_SYSHALT, 0, 0, 0)

//*****************************************************************************
//
// This function sends a system reset command.
//
//*****************************************************************************
#define CANSystemReset()                                                      \
        CANSendMessage(CAN_MSGID_API_SYSRST, 0, 0, 0)

//*****************************************************************************
//
// This function sends a device ID assignment command.
//
//*****************************************************************************
#define CANAssign(ucID)                                                       \
        CANSendMessage(CAN_MSGID_API_DEVASSIGN, 1, (ucID) & 0x3f, 0)

//*****************************************************************************
//
// This function sends a device enumeration command.
//
//*****************************************************************************
#define CANEnumerate()                                                        \
        CANSendMessage(CAN_MSGID_API_ENUMERATE, 0, 0, 0)

//*****************************************************************************
//
// This function sends a synchronous update command.
//
//*****************************************************************************
#define CANSyncUpdate(ucGroup)                                                \
        CANSendMessage(CAN_MSGID_API_SYNC, 1, (ucGroup) & 0xff, 0)

//*****************************************************************************
//
// The function sends a firmware version request to the current device ID.
//
//*****************************************************************************
#define CANFirmwareVersion()                                                  \
        CANSendMessage(CAN_MSGID_API_FIRMVER | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a voltage mode enable command to the current device ID.
//
//*****************************************************************************
#define CANVoltageModeEnable()                                                \
        CANSendMessage(LM_API_VOLT_EN | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a voltage mode disable command to the current device ID.
//
//*****************************************************************************
#define CANVoltageModeDisable()                                               \
        CANSendMessage(LM_API_VOLT_DIS | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a voltage set command to the current device ID.
//
//*****************************************************************************
#define CANVoltageSet(sVoltage, ucGroup)                                      \
        CANSendMessage(LM_API_VOLT_SET | g_ulCurrentID, 3,                    \
                       ((sVoltage) & 0xffff) | (((ucGroup) & 0xff) << 16), 0)

//*****************************************************************************
//
// This function sends a voltage ramp rate set command to the current device
// ID.
//
//*****************************************************************************
#define CANVoltageRampSet(usRamp)                                             \
        CANSendMessage(LM_API_VOLT_SET_RAMP | g_ulCurrentID, 2,               \
                       ((usRamp) & 0xffff), 0)

//*****************************************************************************
//
// This function sends a speed mode enable command to the current device ID.
//
//*****************************************************************************
#define CANSpeedModeEnable()                                                  \
        CANSendMessage(LM_API_SPD_EN | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a speed mode disable command to the current device ID.
//
//*****************************************************************************
#define CANSpeedModeDisable()                                                 \
        CANSendMessage(LM_API_SPD_DIS | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a speed set command to the current device ID.
//
//*****************************************************************************
#define CANSpeedSet(lSpeed, ucGroup)                                          \
        CANSendMessage(LM_API_SPD_SET | g_ulCurrentID, 5, (lSpeed),           \
                       (ucGroup) & 0xff)

//*****************************************************************************
//
// This function sends a speed controller P gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANSpeedPGainSet(lPGain)                                              \
        CANSendMessage(LM_API_SPD_PC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a speed controller I gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANSpeedIGainSet(lPGain)                                              \
        CANSendMessage(LM_API_SPD_IC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a speed controller D gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANSpeedDGainSet(lPGain)                                              \
        CANSendMessage(LM_API_SPD_DC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a speed controller reference command to the current
// device ID.
//
//*****************************************************************************
#define CANSpeedRefSet(ucRef)                                                 \
        CANSendMessage(LM_API_SPD_REF | g_ulCurrentID, 1, ((ucRef) & 0xff),   \
                       0)

//*****************************************************************************
//
// This function sends a position mode enable command to the current device ID.
//
//*****************************************************************************
#define CANPositionModeEnable()                                               \
        CANSendMessage(LM_API_POS_EN | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a position mode disable command to the current device
// ID.
//
//*****************************************************************************
#define CANPositionModeDisable()                                              \
        CANSendMessage(LM_API_POS_DIS | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a position set command to the current device ID.
//
//*****************************************************************************
#define CANPositionSet(lPosition, ucGroup)                                    \
        CANSendMessage(LM_API_POS_SET | g_ulCurrentID, 5, (lPosition),        \
                       (ucGroup) & 0xff)

//*****************************************************************************
//
// This function sends a position controller P gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANPositionPGainSet(lPGain)                                           \
        CANSendMessage(LM_API_POS_PC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a position controller I gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANPositionIGainSet(lPGain)                                           \
        CANSendMessage(LM_API_POS_IC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a position controller D gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANPositionDGainSet(lPGain)                                           \
        CANSendMessage(LM_API_POS_DC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a position controller reference command to the current
// device ID.
//
//*****************************************************************************
#define CANPositionRefSet(ucRef)                                              \
        CANSendMessage(LM_API_POS_REF | g_ulCurrentID, 1, ((ucRef) & 0xff),   \
                       0)

//*****************************************************************************
//
// This function sends a current mode enable command to the current device ID.
//
//*****************************************************************************
#define CANCurrentModeEnable()                                                \
        CANSendMessage(LM_API_ICTRL_EN | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a current mode disable command to the current device ID.
//
//*****************************************************************************
#define CANCurrentModeDisable()                                               \
        CANSendMessage(LM_API_ICTRL_DIS | g_ulCurrentID, 0, 0, 0)

//*****************************************************************************
//
// This function sends a current set command to the current device ID.
//
//*****************************************************************************
#define CANCurrentSet(lCurrent, ucGroup)                                      \
        CANSendMessage(LM_API_ICTRL_SET | g_ulCurrentID, 5, (lCurrent),       \
                       (ucGroup) & 0xff)

//*****************************************************************************
//
// This function sends a current controller P gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANCurrentPGainSet(lPGain)                                            \
        CANSendMessage(LM_API_ICTRL_PC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a current controller I gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANCurrentIGainSet(lPGain)                                            \
        CANSendMessage(LM_API_ICTRL_IC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a current controller D gain set command to the current
// device ID.
//
//*****************************************************************************
#define CANCurrentDGainSet(lPGain)                                            \
        CANSendMessage(LM_API_ICTRL_DC | g_ulCurrentID, 4, (lPGain), 0)

//*****************************************************************************
//
// This function sends a firmware update start command to the current device
// ID.
//
//*****************************************************************************
#define CANUpdateStart()                                                      \
        CANSendMessage(CAN_MSGID_API_UPDATE, 1, g_ulCurrentID & 0xff, 0)

//*****************************************************************************
//
// This function sends a boot loader ping command.
//
//*****************************************************************************
#define CANUpdatePing()                                                       \
        CANSendMessage(LM_API_UPD_PING, 0, 0, 0)

//*****************************************************************************
//
// This function sends a boot loader download command.
//
//*****************************************************************************
#define CANUpdateDownload(ulAddr, ulSize)                                     \
        CANSendMessage(LM_API_UPD_DOWNLOAD, 8, ulAddr, ulSize)

//*****************************************************************************
//
// This function sends a boot loader send data command.
//
//*****************************************************************************
#define CANUpdateSendData(ulSize, ulData1, ulData2)                           \
        CANSendMessage(LM_API_UPD_SEND_DATA, ulSize, ulData1, ulData2)

//*****************************************************************************
//
// This function sends a boot loader reset command.
//
//*****************************************************************************
#define CANUpdateReset()                                                      \
        CANSendMessage(LM_API_UPD_RESET, 0, 0, 0)

//*****************************************************************************
//
// The bit positions of the flags in g_ulStatusFlags.
//
//*****************************************************************************
#define STATUS_FLAG_VOUT        0
#define STATUS_FLAG_VBUS        1
#define STATUS_FLAG_CURRENT     2
#define STATUS_FLAG_TEMP        3
#define STATUS_FLAG_POS         4
#define STATUS_FLAG_SPEED       5
#define STATUS_FLAG_LIMIT       6
#define STATUS_FLAG_POWER       7
#define STATUS_FLAG_FAULT       8
#define STATUS_FLAG_FIRMVER     9
#define STATUS_ENABLED          31

//*****************************************************************************
//
// Prototypes.
//
//*****************************************************************************
extern unsigned long g_pulStatusEnumeration[2];
extern unsigned long g_ulStatusFlags;
extern long g_lStatusVout;
extern unsigned long g_ulStatusVbus;
extern long g_lStatusCurrent;
extern unsigned long g_ulStatusTemperature;
extern long g_lStatusPosition;
extern long g_lStatusSpeed;
extern unsigned long g_ulStatusLimit;
extern unsigned long g_ulStatusPower;
extern unsigned long g_ulStatusFault;
extern unsigned long g_ulStatusFirmwareVersion;
extern unsigned long g_ulCurrentID;
extern volatile unsigned long g_ulCANUpdateAck;
extern void CANCommInit(void);
extern unsigned long CANSendMessage(unsigned long ulMsgID,
                                    unsigned long ulDataLen,
                                    unsigned long ulData1,
                                    unsigned long ulData2);
extern void CANSetID(unsigned long ulID);
extern void CAN0IntHandler(void);
extern void CANTick(void);
extern void CANStatusEnable(void);
extern void CANStatusDisable(void);

#endif // __CAN_COMM_H__
