//*****************************************************************************
//
// controller.h - Prototypes for the motor controller.
//
// Copyright (c) 2008 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
// 
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 3618 of the RDK-BDC Firmware Package.
//
//*****************************************************************************

#ifndef __CONTROLLER_H__
#define __CONTROLLER_H__

//*****************************************************************************
//
// The values that indicate the type of communication link that has been
// detected.
//
//*****************************************************************************
#define LINK_TYPE_NONE          0
#define LINK_TYPE_SERVO         1
#define LINK_TYPE_CAN           2

//*****************************************************************************
//
// The values that indicate the source of a fault.
//
//*****************************************************************************
#define FAULT_SOURCE_CURRENT    1
#define FAULT_SOURCE_TEMP       2
#define FAULT_SOURCE_VBUS       4

//*****************************************************************************
//
// The values that indicate the source from which motor speed is measured.
//
//*****************************************************************************
#define SPEED_SRC_ENCODER       0

//*****************************************************************************
//
// The values that indicate the source from which motor position is measured.
//
//*****************************************************************************
#define POSITION_SRC_ENCODER    0
#define POSITION_SRC_POT        1

//*****************************************************************************
//
// Prototypes for the functions exported by the motor controller.
//
//*****************************************************************************
extern void ControllerLinkGood(unsigned long ulType);
extern void ControllerLinkLost(void);
extern unsigned long ControllerLinkType(void);
extern void ControllerFaultSignal(unsigned long ulFault);
extern unsigned long ControllerFaultsActive(void);
extern long ControllerVoltageGet(void);
extern void ControllerVoltageModeSet(unsigned long bEnable);
extern void ControllerVoltageSet(long lVoltage);
extern void ControllerVoltageRateSet(unsigned long ulRate);
extern void ControllerSpeedModeSet(unsigned long bEnable);
extern void ControllerSpeedSet(long lSpeed);
extern long ControllerSpeedGet(void);
extern void ControllerSpeedSrcSet(unsigned long ulSrc);
extern void ControllerSpeedPGainSet(long lPGain);
extern void ControllerSpeedIGainSet(long lIGain);
extern void ControllerSpeedDGainSet(long lDGain);
extern void ControllerPositionModeSet(unsigned long bEnable,
                                      long lStartingPosition);
extern void ControllerPositionSet(long lPosition);
extern long ControllerPositionGet(void);
extern void ControllerPositionSrcSet(unsigned long ulSrc);
extern void ControllerPositionPGainSet(long lPGain);
extern void ControllerPositionIGainSet(long lIGain);
extern void ControllerPositionDGainSet(long lDGain);
extern void ControllerCurrentModeSet(unsigned long bEnable);
extern void ControllerCurrentSet(long lCurrent);
extern void ControllerCurrentPGainSet(long lPGain);
extern void ControllerCurrentIGainSet(long lIGain);
extern void ControllerCurrentDGainSet(long lDGain);
extern void ControllerInit(void);
extern void ControllerIntHandler(void);

#endif // __CONTROLLER_H__
